/* addcomments.c -- add comments to a NetCDF file */
#include <stdio.h>
#include <netcdf.h>
#include <string.h>
#include <stdlib.h>

#include "nctools.h"

#define MAX_COMMENTS 100

static int ncstatus;
#define handle_error(a) if ((ncstatus = (a)) != NC_NOERR) { \
    fprintf(stderr, "NetCDF error on line %d: %s\n", __LINE__, \
      nc_strerror(ncstatus)); nc_close(ncid); exit(-1); \
} 

static
void
Usage(char **argv)
{
  fprintf(stderr, "Usage: %s [options] file1.nc [file2.nc ...]\n", argv[0]);
  fprintf(stderr, "Options:\n"
	  "       -global s        append s to global \"" COMMENT_NAME "\" attribute\n"
	  "       +global s        overwrite global \"" COMMENT_NAME "\" attribute with s\n"
	  "          -var s1 s2    append s2 to \"" COMMENT_NAME "\" attribute of variable s1\n"
	  "          +var s1 s2    overwrite \"" COMMENT_NAME "\" attribute of variable s1 with s2\n"
	  "   -global_att s1 s2    append s2 to global attribute s1\n"
	  "   +global_att s1 s2    overwrite global attribute s1 with s2\n"
	  "      -var_att s1 s2 s3 append s3 to attribute s1 of variable s2\n" 
	  "      +var_att s1 s2 s3 overwrite attribute s1 of variable s2 with s3\n"
	  " -global_float s f      set global attribute s to float f\n"
	  " -global_short s d      set global attribute s to short integer d\n"
	  "   -global_int s d      set global attribute s to integer d\n"
	  "    -var_float s1 s2 f  set attribute s1 of variable s2 to float f\n"
	  "    -var_short s1 s2 f  set attribute s1 of variable s2 to short integer d\n"
	  "      -var_int s1 s2 f  set attribute s1 of variable s2 to integer d\n"
	  "    -nohistory          do not change the \"history\" global attribute\n"
	  "      -verbose          display files being acted on\n"
	  "         -user s        name (and email address) of the user\n");
}

int
main(int argc, char **argv)
{
  char *comments[MAX_COMMENTS];
  char *variables[MAX_COMMENTS];
  char *attributes[MAX_COMMENTS];
  char overwrite[MAX_COMMENTS];
  float float_att[MAX_COMMENTS];
  int int_att[MAX_COMMENTS];
  short short_att[MAX_COMMENTS];
  nc_type data_type[MAX_COMMENTS];
  char *user = NULL;
  char verbose = 0;
  int ncomments = 0;

  int iarg, ifile;

  char dohistory = 1;

  /* Parse switches */
  iarg=1;
  while (iarg < argc && (*argv[iarg] == '-' || *argv[iarg] == '+')) {
    data_type[ncomments] = NC_CHAR;
    overwrite[ncomments] = 1;
    if (strcmp(argv[iarg], "-global") == 0 || strcmp(argv[iarg], "+global") == 0) {
      if (argc <= iarg+1) {
	fprintf(stderr, "Error: global comment not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	if (*argv[iarg] == '+') {
	  overwrite[ncomments] = 1;
	}
	else {
	  overwrite[ncomments] = 0;
	}
	attributes[ncomments] = NULL;
	variables[ncomments] = NULL;
	comments[ncomments++] = argv[++iarg];
      }
    }
    else if (strcmp(argv[iarg], "-var") == 0 || strcmp(argv[iarg], "+var") == 0) {
      if (argc <= iarg+2) {
	fprintf(stderr, "Error: comment and/or variable name not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	if (*argv[iarg] == '+') {
	  overwrite[ncomments] = 1;
	}
	else {
	  overwrite[ncomments] = 0;
	}
	attributes[ncomments] = NULL;
	variables[ncomments] = argv[++iarg];
	comments[ncomments++] = argv[++iarg];
      }
    }
    else if (strcmp(argv[iarg], "-global_att") == 0 || strcmp(argv[iarg], "+global_att") == 0) {
      if (argc <= iarg+2) {
	fprintf(stderr, "Error: attribute name and/or contents not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	if (*argv[iarg] == '+') {
	  overwrite[ncomments] = 1;
	}
	else {
	  overwrite[ncomments] = 0;
	}
	attributes[ncomments] = argv[++iarg];
	variables[ncomments] = NULL;
	comments[ncomments++] = argv[++iarg];
      }
    }
    else if (strcmp(argv[iarg], "-var_att") == 0 || strcmp(argv[iarg], "+var_att") == 0) {
      if (argc <= iarg+3) {
	fprintf(stderr, "Error: attribute name, variable name and/or contents not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	if (*argv[iarg] == '+') {
	  overwrite[ncomments] = 1;
	}
	else {
	  overwrite[ncomments] = 0;
	}
	attributes[ncomments] = argv[++iarg];
	variables[ncomments] = argv[++iarg];
	comments[ncomments++] = argv[++iarg];
      }
    }
    else if (strncmp(argv[iarg]+1, "global_", 7) == 0) {
      if (argc <= iarg+2) {
	fprintf(stderr, "Error: attribute name and/or contents not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	char *requested_type = argv[iarg]+8;
	int status = 0;

	attributes[ncomments] = argv[++iarg];
	variables[ncomments] = NULL;
	comments[ncomments] = argv[++iarg];

	if (strcmp(requested_type, "float") == 0) {
	  data_type[ncomments] = NC_FLOAT;
	  status = sscanf(comments[ncomments], "%f", &float_att[ncomments]);
	}
	else if (strcmp(requested_type, "short") == 0) {
	  data_type[ncomments] = NC_SHORT;
	  status = sscanf(comments[ncomments], "%hd", &short_att[ncomments]);
	}
	else if (strcmp(requested_type, "int") == 0) {
	  data_type[ncomments] = NC_INT;
	  status = sscanf(comments[ncomments], "%d", &int_att[ncomments]);
	}
	else {
	  fprintf(stderr, "Error: unknown data type \"%s\"\n", requested_type);
	  exit(1);
	}
	
	if (!status) {
	  fprintf(stderr, "Error reading %s as a number\n", comments[ncomments]);
	  exit(1);
	}
	ncomments++;
      }      
    }
    else if (strncmp(argv[iarg]+1, "var_", 4) == 0) {
      if (argc <= iarg+3) {
	fprintf(stderr, "Error: attribute name, variable name and/or contents not specified\n");
	exit(-1);
      }
      else if (ncomments >= MAX_COMMENTS) {
	fprintf(stderr, "Error: no more than %d comments can be added\n",
		MAX_COMMENTS);
	exit(1);
      }
      else {
	char *requested_type = argv[iarg]+5;
	int status = 0;

	attributes[ncomments] = argv[++iarg];
	variables[ncomments] = argv[++iarg];
	comments[ncomments] = argv[++iarg];

	if (strcmp(requested_type, "float") == 0) {
	  data_type[ncomments] = NC_FLOAT;
	  status = sscanf(comments[ncomments], "%f", &float_att[ncomments]);
	}
	else if (strcmp(requested_type, "short") == 0) {
	  data_type[ncomments] = NC_SHORT;
	  status = sscanf(comments[ncomments], "%hd", &short_att[ncomments]);
	}
	else if (strcmp(requested_type, "int") == 0) {
	  data_type[ncomments] = NC_INT;
	  status = sscanf(comments[ncomments], "%d", &int_att[ncomments]);
	}
	else {
	  fprintf(stderr, "Error: unknown data type \"%s\"\n", requested_type);
	  exit(1);
	}
	
	if (!status) {
	  fprintf(stderr, "Error reading %s as a number\n", comments[ncomments]);
	  exit(1);
	}
	ncomments++;
      }      
    }
    else if (strcmp(argv[iarg],"-user") == 0) {
      if (argc > iarg+1) {
	if (user) {
	  fprintf(stderr, "Warning: user specified more than once:"
		  " `%s' not used\n", user);
	}
	user = argv[++iarg];
      }
      else {
	fprintf(stderr, "Error: user not specified\n");
	exit(1);
      }
    }
    else if (strcmp(argv[iarg], "-nohistory") == 0) {
      dohistory = 0;
    }
    else if (strcmp(argv[iarg], "-verbose") == 0) {
      verbose = 1;
    }
    ++iarg;
  }
  if (iarg > argc-1) {
    Usage(argv);
    exit(0);
  }

  if (!ncomments) {
    fprintf(stderr, "Warning: no action taken (no comments given)\n");
    exit(0);
  }

  for (ifile = iarg; ifile < argc; ++ifile) {
    int ncid;
    int j;
    char comments_added = 0;

    handle_error(nc_open(argv[ifile], NC_WRITE | NC_NOCLOBBER, &ncid));
    handle_error(nc_redef(ncid));

    if (verbose) {
      fprintf(stderr, "%s:", argv[ifile]);
    }
    
    for (j = 0; j < ncomments; j++) {
      int varid = NC_GLOBAL;
      char skip_var = 0;
      if (variables[j]) {
	if (nc_inq_varid(ncid, variables[j], &varid) != NC_NOERR) {
	  fprintf(stderr, "Warning: %s not found in %s\n",
		  variables[j], argv[ifile]);
	  skip_var = 1;
	}
      }
      if (!skip_var) {
	size_t attlen = 0;
	char *attribute = attributes[j];

	if (!attribute) {
	  /* Existing comments might be in "comments" or "comment" */
	  if (nc_inq_attlen(ncid, varid, "comments", &attlen) == NC_NOERR) {
	    attribute = "comments";
	  }
	  else if (nc_inq_attlen(ncid, varid, "comment", &attlen) == NC_NOERR) {
	    attribute = "comment";
	  }
	  else {
	    /* Use default held in nctools.h */
	    attribute = COMMENT_NAME;
	  }
	}
	if (data_type[j] == NC_CHAR) {
	  if (overwrite[j]) {
	    handle_error(nc_put_att_text(ncid, varid, attribute, strlen(comments[j]), comments[j]));
	  }
	  else {
	    nct_append_attribute(ncid, varid, attribute, comments[j], "\n");
	  }
	}
	else if (data_type[j] == NC_FLOAT) {
	  handle_error(nc_put_att_float(ncid, varid, attribute, NC_FLOAT, 1, &float_att[j]));
	}
	else if (data_type[j] == NC_SHORT) {
	  handle_error(nc_put_att_short(ncid, varid, attribute, NC_SHORT, 1, &short_att[j]));
	}
	else if (data_type[j] == NC_INT) {
	  handle_error(nc_put_att_int(ncid, varid, attribute, NC_INT, 1, &int_att[j]));
	}
	comments_added = 1;
	if (verbose && varid == NC_GLOBAL) {
	  fprintf(stderr, " %s \".%s\"", overwrite[j] ? "added" : "appended", attribute);
	}
	else if (verbose) {
	  fprintf(stderr, " %s \"%s.%s\"", overwrite[j] ? "added" : "appended", variables[j], attribute);
	}
      }
    }
    if (comments_added && dohistory) {
      nct_add_history(ncid, "Comments added", user);
    }

    handle_error(nc_close(ncid));    
  }

  if (verbose) {
    fprintf(stderr, "\n");
  }

  exit(0);
}
