/* makesum.c -- generates summary files from .dat files
   The summary files produced by this program have a few advantages 
   over the files produced by the fortran program at Chilbolton:

   * The number of scans in the penultimate column is correct 
     (allows for aborted rasters)
   * Range gate spacing is correct for cloud data
   * The date and time are padded by zeros - e.g. now get 01/01/00 
     instead of 1/ 1/ 0
   * Use fseek rather than reading in blocks - should be quicker

   Copyright (C) 2000 Robin Hogan
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "chil.h"

#define SUM_LINE_LENGTH 100

FILE *sum_file = NULL;

void
flush_and_exit(int sig)
{
  if (sum_file) {
    fclose(sum_file);
  }
  exit(1);
}

void
usage(char *program_name)
{
  CHILPrintf("Usage: %s [options] file1.dat [file2.dat ...]\n"
	     "This program creates radar summary files\n"
	     "Options:\n"
	     "     -echo    also print the summary information on standard output\n"
	     "     -Zcal f  specify Z calibration as f dBZ\n"
	     "   -ZDRcal f  specify ZDR calibration as f dB\n"
	     "        -h    display this message\n", program_name);
  exit(0);
}

int
main(int argc, char **argv)
{
  FILE *tmp;
  int file=1, status;
  char *datname, *sumname, *datbase;
  int len, block = 1;
  char sumline[SUM_LINE_LENGTH];
  char oper[4] = "   ";
  int ic;

  int nscans;

  int values[2] = {0, 0};

  float Zcal = 0.0;
  int ZDRcal = 0; /* in 1/8 dB units */

  ChilboltonFile chil_file;
  RasterInfo raster_info;

  char scan_types[] = "URPSFCT";
  char scan_type = 'F';

  char echo = 0;

  signal(SIGINT, flush_and_exit);
  signal(SIGTERM, flush_and_exit);

  /* Parse options */
  if (argc < 2) {
    usage(argv[0]);
  }
  while (file < argc && *argv[file] == '-') {
    if (strcmp(argv[file], "-echo") == 0) {
      echo = 1;
    }
    else if (strcmp(argv[file], "-Zcal") == 0 && file < argc+1) {
      Zcal = atof(argv[++file]);
    }
    else if (strcmp(argv[file], "-ZDRcal") == 0 && file < argc+1) {
      ZDRcal = (int) (0.5 + 8.0 * atof(argv[++file]));
    }
    else {
      usage(argv[0]);
    }
    ++file;
  }

  for (; file < argc; file++)
    {
      datname = argv[file];
      status = CHILOpen(datname, &chil_file);
      block = 1;
      if (status == FILE_CLOSED)
	{
	  CHILPrintf("Error in opening %s\n", datname);
	  exit(1);
	}
      if (chil_file.is_raw)
	{
	  CHILPrintf("%s is raw: skipping\n", datname);
	  CHILClose(&chil_file);
	  continue;
	}

      status = CHILReadInformationRecord(&chil_file, &raster_info);

      if (status == END_OF_DATA)
	{
	  CHILPrintf("Skipping %s: not a Chilbolton-format data file?\n",
		     datname);
	  CHILClose(&chil_file);
	  continue;
	}

      datbase = datname+strlen(datname);
      len = 0;
      while(datbase != datname && datbase[-1] != '/' && datbase[-1] != '\\')
	{
	  --datbase;
	  ++len;
	}

      if (len > 3 && datbase[len-4] == '.' && datbase[len-3] == 'd'
	  && datbase[len-2] == 'a' && datbase[len-1] == 't')
	{
	  sumname = strdup(datbase);
	  if (sumname == NULL)
	    {
	      CHILClose(&chil_file);
	      CHILPrintf("Out of memory\n");
	      exit(1);
	    }
	  strcpy(sumname+len-3, "sum");
	}
      else
	{
	  sumname = malloc(len+4);
	  if (sumname == NULL)
	    {
	      CHILClose(&chil_file);
	      CHILPrintf("Out of memory\n");
	      exit(1);
	    }
	  strncpy(sumname, datbase, len);
	  strcpy(sumname+len, ".sum");
	}

      sum_file = fopen(sumname, "w");
      if (sum_file == NULL)
	{
	  CHILPrintf("Error writing %s\n", sumname);
	  CHILClose(&chil_file);
	  free(sumname);
	  exit(1);
	}
      CHILPrintf("Writing %s\n", sumname);
      fprintf(sum_file, " %s                              %04d\n",
	      datbase, raster_info.file);

      while(1)
	{
	  scan_type = scan_types[0];
	  if (raster_info.scan_type > 0 && raster_info.scan_type <= TS_93) {
	    scan_type = scan_types[raster_info.scan_type];
	  }
	  else {
	    scan_type = scan_types[0];
	  }

	  for (ic = 0; ic < 3; ic++) {
	    oper[ic] = raster_info.oper[ic];
	    if (oper[ic] < ' ' || oper[ic] > '~') {
	      oper[ic] = ' ';
	    }
	  }

	  nscans = -1;
	  if (CHILSkipScansAndCount(&chil_file, &raster_info, &nscans)
	      == CHIL_ERROR)
	    {
	      break;
	    }

	  snprintf(sumline, SUM_LINE_LENGTH-1,
		   "%5u%4u %02u:%02u:%02u %02u/%02u/%02u"
		   " %c %c %2s %s%5.1f%3d%4d%4d%4d%2d"
		   "%6.1f%6.1f%6.1f%5.1f%3d%2d\n", 
		   block, raster_info.raster,
		   raster_info.hour, raster_info.minute, raster_info.second,
		   raster_info.day, raster_info.month, raster_info.year,
		   'A', scan_type,
		   CHILExperimentCode(&raster_info, NULL), oper,
		   Zcal, ZDRcal, values[0], raster_info.raw[30],
		   raster_info.gate_spacing, values[1],
		   raster_info.min_angle / 60.0, raster_info.max_angle / 60.0,
		   raster_info.init_angle / 60.0, raster_info.angle_inc / 60.0,
		   nscans, raster_info.sens_flag);

	  fprintf(sum_file, sumline);
	  if (echo)
	    {
	      printf(sumline);
	      fflush(stdout);
	    }
	  block = 1 + (ftell(chil_file.file) / CHIL_BLOCKSIZE);
	  status = CHILReadInformationRecord(&chil_file, &raster_info);
	  if (status == END_OF_DATA)
	    {
	      break;
	    }
	}
      CHILClose(&chil_file);
      tmp = sum_file;
      sum_file = NULL;
      fclose(tmp);
      free(sumname);
    }

  exit(0);
}
